"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Telegram = void 0;
const axios_1 = __importDefault(require("axios"));
const form_data_1 = __importDefault(require("form-data"));
const consts_1 = require("../consts");
class Telegram {
    ignoreSendingErrors = false;
    priorityTransaltion = consts_1.priorityTransaltion;
    typeTransaltion = consts_1.typeTransaltion;
    channels = {};
    i18n = {
        priority: "Priority",
        type: "Type"
    };
    i18nTranslation(values) {
        this.i18n = { ...this.i18n, ...values };
    }
    translateTypes(values) {
        this.typeTransaltion = { ...this.typeTransaltion, ...values };
    }
    translatePriorities(values) {
        this.priorityTransaltion = { ...this.priorityTransaltion, ...values };
    }
    configureChannels(channels) {
        this.channels = { ...this.channels, ...channels };
    }
    constructor(opts) {
        this.ignoreSendingErrors = opts.ignoreSendingErrors;
        this.channels.default = {
            chatId: opts.defaultChatId,
            botToken: opts.defaultBotToken,
        };
    }
    #getChatAndTokenByType(type) {
        const channel = this.channels[type];
        let chatId = undefined;
        let botToken = undefined;
        if (channel?.chatId) {
            chatId = this.channels.info?.chatId;
        }
        else if (this.channels.default?.chatId) {
            chatId = this.channels.default?.chatId;
        }
        else {
            throw new Error(`You need to set default channel or ${type} to send ${type} log; please see the documentation!`);
        }
        if (channel?.botToken) {
            botToken = this.channels.info?.botToken;
        }
        else if (this.channels.default?.botToken) {
            botToken = this.channels.default?.botToken;
        }
        else {
            throw new Error(`You need to set default channel or ${type} to send ${type} log; please see the documentation!`);
        }
        return [chatId, botToken];
    }
    async info(msg, staticArgs, ignoreSendingErrors, file) {
        return await this.#sendLog("info", { msg, ignoreSendingErrors, file, staticArgs });
    }
    async debug(msg, staticArgs, ignoreSendingErrors, file) {
        return await this.#sendLog("debug", { msg, ignoreSendingErrors, file, staticArgs });
    }
    async error(msg, staticArgs, ignoreSendingErrors, file) {
        return await this.#sendLog("error", { msg, ignoreSendingErrors, file, staticArgs });
    }
    async success(msg, staticArgs, ignoreSendingErrors, file) {
        return await this.#sendLog("success", { msg, ignoreSendingErrors, file, staticArgs });
    }
    async warning(msg, staticArgs, ignoreSendingErrors, file) {
        return await this.#sendLog("warning", { msg, ignoreSendingErrors, file, staticArgs });
    }
    async #sendLog(type, opts) {
        opts.staticArgs = opts.staticArgs || {};
        opts.staticArgs.type = type;
        const finalMsg = this.formatMessage(opts.msg.trim(), opts.staticArgs);
        let [chatId, botToken] = this.#getChatAndTokenByType(type);
        return await this.send({ ...opts, chatId, token: botToken, msg: finalMsg });
    }
    async send(opts) {
        try {
            if (opts.file) {
                const data = new form_data_1.default();
                data.append('chat_id', opts.chatId);
                data.append('parse_mode', 'HTML');
                data.append('document', opts.file.buff, opts.file.fileName ?? '');
                data.append('caption', opts.msg || '');
                return await axios_1.default.post(`https://api.telegram.org/bot${opts.token}/sendDocument`, data, {
                    headers: data.getHeaders(),
                    'maxContentLength': Infinity,
                    'maxBodyLength': Infinity
                });
            }
            else {
                await axios_1.default.get(`https://api.telegram.org/bot${opts.token}/sendMessage?chat_id=${opts.chatId}&text=${encodeURI(opts.msg)}&parse_mode=HTML`);
            }
        }
        catch (err) {
            if (!this.ignoreSendingErrors || (!opts.ignoreSendingErrors && typeof opts.ignoreSendingErrors != "undefined")) {
                throw err;
            }
            console.error({}, err);
        }
    }
    formatMessage(msg, staticArgs) {
        const newLine = '\n';
        let finalMessage = newLine.repeat(2);
        if (staticArgs) {
            if (staticArgs.priority) {
                finalMessage += `<b>${this.i18n.priority}</b>  : <i>${this.priorityTransaltion[staticArgs.priority] ?? staticArgs.priority}</i> ${newLine}`;
                delete staticArgs.priority;
            }
            if (staticArgs.type) {
                finalMessage += `<b>${this.i18n.type}</b>  : <i>${this.typeTransaltion[staticArgs.type] ?? staticArgs.type}</i> ${newLine}`;
                delete staticArgs.type;
            }
            for (const key of Object.keys(staticArgs)) {
                finalMessage += `<b>${key}</b>  : <i>${staticArgs[key]}</i> ${newLine}`;
            }
        }
        if (finalMessage != "")
            finalMessage += '\n ============================================ \n\n';
        finalMessage += msg.replace(/(?:\r\n|\r|\n)/g, newLine);
        return finalMessage;
    }
}
exports.Telegram = Telegram;
